#!/usr/bin/perl
# $Id: run_furnace.pl,v 1.30 2023/11/17 09:11:41 sqko Exp $
$SIG{INT} = \&my_sigint;
use strict;
use warnings;
use Sys::Hostname;
use SemaforeFile;
use RFC::Furnace;
use RFC::Main;

use Mail::Sender;
my $PROGRAM = $0;
$PROGRAM =~ /([\w\.\-]+)$/;
$PROGRAM = $1;
if (scalar(@ARGV) < 2)
{
    exit;
}
my $mta = &RFC::Main::get_cv('global','mta');
$mta = '' unless ($mta);
$mta =~ /([\w\/\.\-]+)/;
$mta = $1;

my $rig = RFC::Furnace->new(shift);
$rig->check_run;
my $smtp = &RFC::Main::get_cv('global','smtp');
my $file = shift;
if (!-e($file))
{
    $rig->proglog("No file: $file found by $PROGRAM, exiting!",$PROGRAM);
    exit;
}

my $sleeptime = &RFC::Main::get_cv('furnace','sleeptime');
$sleeptime = 5 unless ($sleeptime && $sleeptime > 5); ## Minimum 5 seconds
my $start_time = localtime();
my $email = undef;
my $rid = '';
if (@ARGV) 
{
    $rid = shift;
    $email = RFC::Main::passwd_client('email',$rid) if ($rid);
}

my $f = SemaforeFile->new($file);
my @list = ();
my @prog = ();
foreach my $str ($f->readlines)
{
    chomp $str;
    push @prog,$str;
    next if ($str =~ /^\s*\#/);
    next if ($str =~ /^\s*$/);
    push @list,$str;
}

if ($rig->is_running())
{
    die "A program is already running, program activation aborted!\n";
}

## Unset all warnings for tempcontrol devices ##
foreach my $name ($rig->rig->list_devices('tempcontrol'))
{
    my $dev = $rig->rig->get_device('tempcontrol',$name);
    $rig->rig->unset_warning($dev) if ($dev);
}


my $runnr = 0;
my $linenr = 0;
my $count = 0;
my $str = '';
eval {
    $runnr = $rig->start_program($rid,@prog);
    unless ($runnr)
    {
	### Do cleanup if start program did not work ###
	$rig->stop_program();
	die "Error: Could not start program!\n";
    }
    foreach $str (@list)
    {
	$linenr++;
	$str =~ s/[\r\n]*//g;
	last if ($str =~ /^\s*exit\s*$/);
	unless ($rig->is_running())
	{
	    $rig->proglog("Program terminated as semafore has been removed!",$PROGRAM);
	    last;
	}
	$count++;
	$str =~ s/\,/\./g;
	if ($str =~ /wait\s*\=\s*[\d\.]+/){&wait($str);  next;};
	if ($str =~ /hold\s*\=\s*[\d\.]+/){&wait($str);  next;};
	if ($str =~ /^\s*text\s*\=/) {&cmdl($str);  next;};
	if ($str =~ /^\s*temp\s*\=/) {&temp($str);  next;};
	if ($str =~ /^\s*addjust_temp\s+/) {&addj_temp($str);  next;};
	if ($str =~ /^\s*ramp\s*\=/) {&ramp($str);  next;};
	if ($str =~ /^\s*measure\s*/) {&my_measure;  next;};
	if ($str =~ /^\s*mail:/) {&mail($str); next;};
    }
};
if ($@)
{
    $str = "Runtime error on line $linenr! Command '$str' could not be completed, Errormessage: $@";
    $rig->errorlog($str,$PROGRAM);
    warn $str,"\n";
}
else
{
    $str = "Program finished. $count commands executed!";
    $rig->proglog($str,$PROGRAM);
}

## Get additional recipients ##
my %h = $rig->get_run_info($runnr);
my $email2 = '';
$email2 = RFC::Main::passwd_client('email',$h{'ordered_by'}) if ($h{'ordered_by'});

## Plot the furnace run ###
my $total = 0;
($file,$total) = $rig->get_plot($runnr,'force');
## send email with plot to user ###
my $adr = $email;
my $s = "Furnace run $runnr for $rig finished";
$s  = "Furnace run $runnr for $rig finished prematurely, only $count comands executed!" unless ($rig->is_running());
my $msg = $str;
$msg .= "\n\nFurnace run information:\n".join("\n",$rig->get_info($runnr));
$msg .= "\n\nCommand log:\n".join("\n",$rig->get_proglog($runnr));



eval {
    if ($mta && -e $mta)
    {
	my $username = getpwuid( $< );
	my $adr = $email;
	$adr .= ' '.$email2 if ($email2);
	$adr =~ /([\w\s\.\@\-]+)/;
	$adr = $1;
	$s =~ /([\w\s\.\-\+\:\;\(\)\!\,]+)/;
	$s = $1;
	my $cmd;
	if ($file)
	{
	    $file =~ /([\w\.\/\-]+)/;
	    $file = $1;
	    die "File '$file' does not exists!\n" unless (-e $file);
	    if ($mta =~ /mailx/)
	    {
		$cmd = "| $mta -s '$s' -A $file ".$adr;
	    }
	    else
	    {
		$cmd = "| $mta -s '$s' -a $file ".$adr;
	    }
	}
	else
	{
	    $cmd = "| $mta -s '$s' ".$adr;
	}
	my $pid = open(MAIL,$cmd) or die "Couldnt fork $!\n";
	print MAIL $msg;
	my $res = close(MAIL);
	die "Mail not sent!" unless ($res);
    }
    else
    {
	my $username = getpwuid( $< );
	my $sender = new Mail::Sender {
	    'smtp' => $smtp,
	    'from' => $username.'@'.hostname,
	    'replyto' => $email,
	    'on_errors' => undef
	} or die "Can't create the Mail::Sender object: $Mail::Sender::Error\n";
	if ($email2 && $email2 ne $adr)
	{
	    $sender->OpenMultipart({
		'to' => $adr,
		'cc' => $email2,
		'subject' => $s
				   }) or die "Can't open the message: $sender->{'error_msg'}\n";	
	}
	else
	{
	    $sender->OpenMultipart({
		'to' => $adr,
		'subject' => $s
				   }) or die "Can't open the message: $sender->{'error_msg'}\n";
	}
	$sender->Body;
	$sender->SendLineEnc($msg);
	die "File '$file' does not exists!\n" unless (-e $file);
	my $filename = $file;
	if ($filename && $filename =~ /([\w\.\-\s\:]+)$/)
	{
	    $filename = $1;
	}
	$sender->Attach({
	    'description' => 'File',
	    'disposition' => 'attachment; filename="'.$filename.'"',
	    'file' => $file
			}) or die "Could not atttach file $file: $sender->{'error_msg'}\n";
	$sender->Close() or die "Failed to send the message: $sender->{'error_msg'}\n";
    }
};
if ($@)
{
    $rig->errorlog("Mail could not be sent: $@",$PROGRAM);
}
$rig->stop_program($PROGRAM,'nolog');
$rig = undef;
exit;



sub my_sigint
{
    ### Exits gracefully when catching a sigint (kill -2)!! ###
    $SIG{INT} = 'IGNORE';  ## Ignores further SIGINT ###
    ### Do cleanup ###
    $rig->stop_program();
    if ($email)
    {
	my $message = "Program for furnace ".$rig->name." started $start_time was terminated prematurely!\n";
	my $subject = "Program for furnace ".$rig->name." terminated!";
	RFC::Main::mail($email,$message,$subject);
    }
    $rig->proglog("$0 caught SIGINT, exiting!!!\n",$PROGRAM);
    exit;
}

sub mail($)
{
    my $str = shift;
    if ($str =~ /mail:\s*([\w\.]+\@[\w\.]+)\s+(.*)/)
    {
	my $adr = $1;
	my $mess = $2;
	eval {
	    RFC::Main::mail($adr,$mess,"$PROGRAM for rig ".$rig->name." sends the following message",$PROGRAM);
	    $rig->proglog("Mail sent to $adr\n",$PROGRAM);
	};
	if ($@)
	{
	    $rig->proglog("Mail to $adr could not be sent! Error: $@\n",$PROGRAM);
	}
    }
}


sub my_measure()
{
    &measure();
    sleep(10);
    &measure();
}

sub wait($)
{
    my $string = shift;
    my $time = 0;
    if ($string =~  /wait\s*\=\s*([\d\.]+)/)
    {
	$time = $1*60;
    }
    if ($string =~  /hold\s*\=\s*([\d\.]+)/)
    {
	$time = $1*60*60;
    }
    $rig->proglog(sprintf("Waiting for %.2f hours",$time/3600),$PROGRAM);
    my $end_time = time + $time;
  OUTER:
    while (time < $end_time)
    {
	&measure();
	### Wait between data log ###
	my $et = time + $sleeptime;
	while (time < $et)
	{
	    last OUTER unless ($rig->is_running);
	    sleep(1);
	}
    }
}

sub addj_temp($)
{
    my $string = shift;
    if ($string =~ /^\s*addjust_temp\s+(\d+)\s+(\d+)\s*(\w*)/)
    {
	my $target = $1;
	my $max_offset = $2;
	my $cdevicename = $3;
	my $devname = $rig->get_cv('control_logic','templog_for_temperature_addjust');
	$devname = ($rig->temps)[0] unless ($devname);
	my $device = $rig->temp($devname);
	unless ($device)
	{
	    $rig->proglog("Error: Could not get device for temperature name '$devname'",$PROGRAM);
	    return;
	}
	my $temp = $device->read;
	my $offset = $target - $temp;
	if (abs($offset) > $max_offset)
	{
	    $rig->proglog("Temperature offset ($offset) larger than allowed ($max_offset) for temperature addjustment",$PROGRAM);
	}
	else
	{
	    my @list = $rig->tempcontrols();
	    unless (@list)
	    {
		$rig->proglog("Error: No temperature control devices found!",$PROGRAM);
		&wait('wait = 180');
		return;
	    }
	    my $controldevice;
	    if ($cdevicename)
	    {
		$controldevice = $rig->tempcontrol($cdevicename);
	    }
	    else
	    {
		$controldevice = $rig->tempcontrol($list[0]);
	    }
	    my $ctemp = $controldevice->get_temp;
	    if ($ctemp)
	    {
		$rig->proglog("Addjusting temperature $offset C, Using '$devname' as reference!",$PROGRAM);
		my $newtemp = $ctemp + $offset;
		$controldevice->set_temp($newtemp,$PROGRAM);
	    }
	    else
	    {
		$rig->proglog("Could not get current controler setpoint, aborting temperature addjustment!",$PROGRAM);
	    }
	    &wait('wait = 180');
	}
    }
    else
    {
	$rig->proglog("Not enough arguments for automatic temperature adjustment!",$PROGRAM);
    }
}

sub temp($)
{
    my $string = shift;
    $string =~ /temp\s*\=\s*(\d+)[\s\t]*(\w*)/;
    my $temp = $1+0;
    my $contr = $2;
    my $res = 0;
    if ($contr)
    {
	$res = $rig->set_temp($temp,$contr,$PROGRAM);
    }
    else
    {
	$res = $rig->set_temp($temp);
    }
    ## Append a line to proglog for furnace run as set_temp does not know about this ##
    if (defined($res))
    {
	$rig->proglog("Setting furnace setpoint to $temp C",$PROGRAM);
    }
    else
    {
	$rig->proglog("Error: Could not furnace setpoint to $temp C",$PROGRAM);
    }
}

sub ramp($)
{
    my $string = shift;
    $string =~ /ramp\s*\=\s*([\d\.]+)[\s\t]*(\w*)/;
    my $ramp = $1+0;
    my $contr = $2;
    my $res = 0;
    if ($contr)
    {
	$res = $rig->set_ramp($ramp,$contr,$PROGRAM);
    }
    else
    {
	$res = $rig->set_ramp($ramp);
    }
    ## Append a line to proglog for furnace run as set_ramp does not know about this ##
    if (defined($res))
    {
	$rig->proglog("Setting furnace ramprate to $ramp C/hour",$PROGRAM);	
    }
    else
    {
	$rig->proglog("Error: Could not set furnace ramprate to $ramp C/hour",$PROGRAM);
    }
}

sub cmdl($)
{
    my $string = shift;
    $string =~ /\=\s*([\w\s\.\,\-\+\'\"\;\:\!]+)/;
    my $st = $1;
    $rig->proglog($st,$PROGRAM);
}


#### Utility functions ###

sub get_host($)
{
    my $adr = shift;
    if ($adr =~ /([\d\.\:]+)/)
    {
        my $cmd = &get_cv('impedance','remote_client');
	return undef unless ($cmd);
	$cmd .= " $1 ping";
        my $res = `$cmd`;
        if ($res =~ /localhost.* on addr:\s*([\d\.]+)/)
        {
            return $1;
        }
        if ($res =~ /on ([\w\-\.]+) on addr:/)
        {
            return $1;
        }
    }
    if ($adr =~ /([\w\.]+)/)
    {
        my $ip = $1;
        my $res = `/usr/bin/nslookup $ip`;
        if ($res && $res =~ /name\s+\=\s+([\w\-\+]+)/)
        {
            return $1;
        }
        return $ip;
    }
    return $adr;
}

sub measure()
{
    return $rig->writedata();
}
