#!/usr/bin/perl
# $Id: furnace.cgi,v 1.90 2022/11/08 07:28:06 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2017              ###
###                                                      ###
############################################################
############################################################
use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use Sys::Hostname;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use RFC::Furnace;
my $hostok = &RFC::Furnace::can_start_furnace();
my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
my $furnace = undef;
my $acmode = get_cv('global','access_mode');
$acmode = 'celltest' unless ($acmode);
my $local_admin = 0;
my $admin = 0;
my $equipid = $rig->get_cv('main','equipmentid');
$equipid = 0 unless ($equipid);
my $eqadr = get_cv('global','remote_equipment_database');
$eqadr = '' unless ($eqadr);
my $logaddr = get_cv('global','logbook_url');
my $readmode = undef;
my $usermode = undef;
my $modifymode = undef;
my $eqid = $rig->get_cv('main','equipmentid');
my $remote = $rig->get_cv('main','authorisation');
my @tubes;
my @runmodes;
my $minramp = 0;
my $maxramp = 1e100;
my $mintemp = -273;
my $maxtemp = 600;
my ($u,$p) = &RFC::RFCCGI::get_id();

if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end('../RFCfurnace/manual.pdf');
}
else
{
    if (login_ok())
    { 
	my $id = &RFC::Main::get_cv('global','authorisation_change_allow_start');
#	$local_admin = &RFC::Main::passwd_client('safety_auth',$u,$id) if ($id);
	$local_admin = &RFC::Main::passwd_client('safety_auth_nolabtech',$u,$id) if ($id);
	$admin = &RFC::Main::passwd_client('isadmin',$u,$p);
	unless ($local_admin || $admin)
	{
	    $hostok = 0 if (&RFC::Main::get_cv('global','allow_start') && &RFC::Main::get_cv('global','allow_start') =~ /no/i);
	}
	$readmode = user_auth($acmode,'r');
	$usermode = user_auth($acmode,'c');
	$modifymode = user_auth($acmode,'m');
	if (RFC::RFCCGI::rig_mode())
	{
	    ####  Read and use mode deprecated after initialisation of rig authentification #### 
	    $readmode = 1;
	    $usermode = 1;
	    $modifymode = RFC::RFCCGI::rig_cert() unless ($modifymode);
	}
	unless (RFC::RFCCGI::rig_auth())
	{
	    $readmode = 0;
	    $usermode = 0;
	    $modifymode = 0;
	}
        &print_header("Main page for furnace ".$rig->name,-onload=>'load_start()');
	print '<link rel="stylesheet" type="text/css" href="/RFCfurnace/Furnace.css" />',"\n";
	print '<script src="/RFCfurnace/Furnace.js" type="text/javascript"></script>',"\n";
        print <<END;

<script>

function load_start()
{

}

</script>

END
	
        top_nav_bar_start;
        if ($logaddr && $equipid)
        {
	    top_nav_bar_button($logaddr,'','','Logbook for furnace '.$rig->name,'s3','rig,'.$rig->name.',1',"id,$equipid,1");
	}
        else
        {
	    top_nav_bar_button('../celltest/logbook.cgi','','',"Logbook for furnace ".$rig->name,'s3','rig,'.$rig->name.',1');
	}
        if ($modifymode)
        {
	    top_nav_bar_button('misc_conf.cgi','','','Configuration of furnace '.$rig->name,'s1','rig,'.$rig->name.',1');
	}
        else
        {
	    top_no_button;
	}
        top_nav_bar_button('search.cgi','','',"Show data for Furnace ".$rig->name,'s2','rig,'.$rig->name.',1');
	tab_newrow;
	if (get_CGI_value('action'))
	{
	    top_no_button;
	}
	else
	{
	    &js_back();
	    top_js_return('#006600','white');
	}
	if ($modifymode)
	{
	    top_nav_bar_button('../celltest/rig_main.cgi','','','RFCcontrol setup','s1','rig,'.$rig->name.',1');
	}
	else
	{
	    my $server = get_cv('passwds','passwd_server');
	    my $taskid = &get_cv('safety_task_access','rig'.$rig->name); 
	    my @args = ('col,id,1','rig,'.$rig.',1');
            my $res = &RFC::Main::passwd_client('list_equip_auth',$eqid);
            if ($eqid && $remote && $remote =~ /remote/i && $res && $res != -1)
            {
                push @args,'equipid,'.$eqid.',1';
                foreach (split(/\s*\n\s*/,$res))
                {
                    push @args,'search_val,'.$_.',1';
                }
            }
            else
            {
		foreach (split(/\s*\,\s*/,$taskid))
		{
		    push @args,'search_val,'.$_.',1';
		}
	    }
	    if (CGI::https())
	    {
		top_nav_bar_button('https://'.$server.'/cgi-bin/search_safety_task.cgi','Search','dummy','Show nescesarry authorisations','s1',@args);
	    }
	    else
	    {
		top_nav_bar_button('http://'.$server.'/cgi-bin/search_safety_task.cgi','Search','dummy','Show nescesarry authorisations','s1',@args);
	    }
	}
        if ($modifymode)
        {
	    my $r = $rig->name();
	    my $test = $rig->session();
	    my $server = hostname;
	    my $url = "http://$server/rig$r/".$r."test$test/errorlog";
	    $url = "https://$server/rig$r/".$r."test$test/errorlog" if (CGI::https());
	    print '<td>';
	    print start_form(-action=>'/cgi-bin/show_data.cgi',-name=>'errorlog');
	    &print_hidden();
	    print hidden(-name=>'url',-value=>$url);
	    print end_form();
	    print button(-name=>'Show errorlog',-class=>'s3',-onclick=>'document.errorlog.submit()');	    
	    print '</td>';
        }        
        print '<td>';
        if ($rig->get_cv('main','allowed_tubes'))
        {
	    print button(-name=>'Show seen elements',-id=>'tube_e',-class=>'hidden',-onclick=>'show_tube_elements()');	    
	}
        else
        {
	    print button(-name=>'Show seen elements',-id=>'tube_e',-class=>'s3',-onclick=>'show_furnace_elements()');
	}
        print '</td>';
	top_nav_bar_button('main.cgi','select_server','','To menu','s4');
	end_top_bar;
	    eval {
	    $furnace = RFC::Furnace->new($rig);
	    @tubes = $furnace->list_tubes();
	    if ($rig->get_device('relay','furnace_mode'))
	    {
		@runmodes = $rig->get_cv('main','run_modes');
	    }
        };
        if ($@)
        {
	    print '<center>';
	    print h1({-class=>'Alert'},$@);
            print h1("You likely have to configure the underlying RFC::Rig instance to correct this problem");
	    print '</center>';
        }
        else
        {
	    param(-name=>'action',-value=>'')  unless (defined(param('action')));
	    $_ = param('action');
	  CASE:
	    {
		### Insert case statements ###
		/set_elements/ and do {&set_elements();};
		/set_temp/ and do {&set_temp();};
		/set_ramp/ and do {&set_ramp();};
		print '<div class="body_div">';
		&main_page;
		
	    }
	}
        &print_end('../RFCfurnace/manual.pdf');
		print '</div>';
		print '</div>';
    }
    else 
    {
	&not_auth();
    }
}

$rig = undef;    
exit;

sub set_elements()
{
    my $user = &get_user();
    my $str = param('f_elements');
    return unless ($str);
    my @list = ();
    my @elements = RFC::Furnace::get_element_data();
    foreach my $n (split(/\s*\,\s*/,$str))
    {
	foreach my $e (@elements)
	{
	    next unless ($$e{'short'} eq $n);
	    push @list,$$e{'Z'};
	    last;
	}
    }
    my @curr = $furnace->list_elements();
    ### ADD missing ###
    foreach (@list)
    {
	$furnace->add_element($_,$user);
    }
    ## Remove deleted ##
  OUTER:
    foreach my $o (@curr)
    {
	foreach my $i (@list)
	{
	    next OUTER if ($i == $o);
	}
	$furnace->del_element($o,$user);
    }
}

sub set_ramp()
{
    return unless ($modifymode);
    my $temp = get_CGI_value_clean('ramp');
    if ($temp && $temp =~ /([\d\.\-\+]+)/)
    {
	$temp = $1;
	$furnace->ramp($temp,&get_user());
	print center(h2("Furnace ramprate set to $temp C/hour"));
    }
}

sub set_temp()
{
    return unless ($modifymode);
    my $temp = get_CGI_value_clean('temp');
    if ($temp && $temp =~ /([\d\.]+)/)
    {
	$temp = $1;
	$furnace->temp($temp,&get_user());	    
	print center(h2("Furnace setpoint set to $temp C"));
    }
}

sub main_page
{
    my $atemp = sprintf("%.1f",$furnace->get_atemp());
    my $temp = $furnace->temp();
    my $ramp = $furnace->ramp();
    $maxramp = $furnace->maxramp();
    $minramp = $furnace->minramp();
    $maxtemp = $furnace->maxtemp();
    $mintemp = $furnace->mintemp();
    my $runnr = 0;
    my $status = $furnace->rig->get_cv('main','status');
    print '<div class=container_div>';
    print '<div class="heading_div">';
    if (@tubes)
    {
	print h1("Tube $furnace");
    }
    else
    {
	print h1("Chamber $furnace");
    }
    print '</div>';
    print '<div class="heading_right_div">';
    print "<tr><th><h2> Current temperature: $atemp C </h2></th></tr>";
    print '</div>';
    print '</div>'; 
    if ($status && $status !~ /^OK/i)
    {
	print h1({-class=>'Alert'},$status.'!');
    }
    if ($furnace->is_running())
    {
	$runnr = $furnace->runnr();
	print '<div class="heading_div alert" style="width:25%;">';
	print '<h2> Furnace is on! </h2>';
	print '</div>';
	print '<center>';
	print '<div class="furnace_div">';
	print '<table class="furnace_main">',"\n";
	print '<tr><td>',"\n";
	print '<table class="program">';
#	print '<tr><th colspan=3>';
#	print 'Program information:</th></tr>';
	print '<tr><th colspan=3 id="totaltime">';
	print '<tr><td colspan=3>';
	print textarea(-name=>'info',-id=>'info',class=>'proglog',-rows=>6,-readonly=>1);
	print '</td></tr>';

	print '<tr><th colspan=3>';
	print 'Program log:</th></tr>';
	print '<tr><td colspan=3>';
	print textarea(-name=>'log',-id=>'log',class=>'proglog',-rows=>6,-readonly=>1);
	print '</td></tr>';

	print '<tr><th colspan=3>';
	print 'Add comment:</th></tr>';
	print '<tr><td colspan=3>';
	print start_form(-name=>'new_comment');
	print hidden(-name=>'eqid',-value=>$equipid,-force=>1);
	print hidden(-name=>'addr',-value=>$eqadr,-force=>1);
	print textarea(-name=>'comments',-value=>'',-force=>'1',-rows=>'3',-class=>'textfield');
	print button(-name=>'new_comm',-value=>'Add comment to logbook',-class=>'s3',-onclick=>"add_comment($runnr)");
	print end_form();
	print '</td></tr>';


	print '<tr><td></td><td><center>';
	print button(-name=>'Stop program',-class=>'stop_program',-onclick=>'stop_program()',-id=>'stop') if ($readmode && $hostok);
	print '</center></td><td>';
	print '</td></tr>';
	print '</table>';
	print '</td>';
	print '<td>';
	print '<img id="profile" src="/RFCfurnace/dummy.png"></img>';
	print '</td></tr>',"\n";
	print '</table>';
	print '</div>';
	print '</div>';
	print '</center>';
    }
    else
    {
	if ($readmode && $hostok)
	{
	#    print '<table class=furnace_main>',"\n";
	#    print '<tr><td>',"\n";
	    print '<div class="container_div left">';
	    print '<div class="furnace_div">';
	    print '<table class=program>';
	    print '<tr><th colspan=3 id="totaltime">';
	    print 'Current program:</th></tr>';

	    print '<tr><td colspan=3>';
	    print textarea(-name=>'program',-id=>'program',class=>'textfield',-rows=>10,-onchange=>'change_program(this.value)');
	    print '</td></tr>';
	    print '<tr><th style="background:#c2c2c254"> Set point (C): </th><td style="background:#c2c2c254">';
	    print textfield(-name=>'setpoint',-id=>'setpoint',-value=>'',-class=>'setpoint');
	    print '</td><td rowspan=2>';
	    print button(-name=>'Add ramp',-class=>'add_setpoint',-onclick=>'add_line_ramp()');
	    print '</td></tr>';
	    print '<tr><th style="background:#c2c2c254"> Ramp rate (C/h): </th><td style="background:#c2c2c254">';
	    print textfield(-name=>'ramprate',-id=>'ramprate',-value=>'',-class=>'setpoint');
	    print '</td></tr>';
	    print '<tr><th> Hold (h): </th><td>';
	    print textfield(-name=>'wait',-id=>'wait',-value=>'',-class=>'setpoint');
	    print '</td><td rowspan=1>';
	    print button(-name=>'Add hold',-class=>'add_setpoint_low',-onclick=>'add_line_hold()');
	    print '</td></tr>';
	    print '<tr><th> </th><td>';

	    print '</td><td rowspan=1>';
	    print button(-name=>'Clear program',-class=>'clear_program',-onclick=>'program.length = 0; update_plot()');
	    print '</td>';
	    print '</tr>';
	    my $minramp = $furnace->minramp();
	    $minramp = 6 unless ($minramp); ## Default minimum ramprate for Eurotherm controlers
	    if ($minramp)
	    {
		my $r1 = $minramp;
		my $r2 = $minramp * 2;
		my $r3 = $minramp * 3;
		my $max = $minramp * 20;
		my $str = "<p><b>Attention!</b> If your want to use a ramp rate of less than $max C/h your ramp rate has to be a multiple of $minramp C/h ($r1, $r2, $r3 etc. C/h) </p>";
		print "<tr><td colspan=3>$str</td></tr>\n";
 	    }
	    print '</table>';
	    print '</td>';
	    print '</div>';
	    print '<div class="furnace_div">';
	    print '<table class=program>';
	    print '<tr><th colspan=2>';
	    print 'Sample information:</th></tr>';
	    print '<tr><th style="width:45%;"> Ordered by (*): </th><td>';
	    print textfield(-name=>'order',-id=>'order',-value=>'',-class=>'textfield');
	    print '</td></tr>';
	    if (@tubes)
	    {
		print '<tr><th> Furnace tube: </th><td>';
		print popup_menu(-name=>'tube',-values=>['',@tubes],-id=>'tube',-onchange=>'get_elements_tube(this.value)',-class=>'textfield');
		print '</td></tr>';
	    }
	    else
	    {
		print hidden(-name=>'tube',-id=>'tube',-value=>-1,-force=>1);
	    }
	    print '<tr><th> Chemical elements (*): </th><td>';
	    print textfield(-name=>'comp',-id=>'comp',-value=>'',-class=>'textfield',-onclick=>'pick_elements(this.value)',-onchange=>'check_comp(this.value)');
	    print '</td></tr>';
	    print '<tr><th> Sample ID: </th><td>';
	    print textfield(-name=>'id',-id=>'id',-value=>'',-class=>'textfield');
	    print '</td></tr>';
	    print '<tr><th> Project: </th><td>';
	    my $validate = &get_cv('global','project_number_validate_url');
	    if ($validate)
	    {
		my $q = '&name='.param('name').'&pass='.param('pass');
		print textfield(-name=>'project',-id=>'project',-value=>'',-class=>'textfield',-onchange=>"raw_ajax('$validate'+this.value,'$q',ajax_alert)");
	    }
	    else
	    {
		print textfield(-name=>'project',-id=>'project',-value=>'',-class=>'textfield');
	    }
	    print '</td></tr>';
	    if (@runmodes)
	    {
		unshift @runmodes,'';
		print '<tr><th> Atmosphere: </th><td>';
		print popup_menu(-name=>'runmode',-id=>'runmode',-default=>'',-class=>'textfield',-values=>[@runmodes]);
		print '</td></tr>';		
	    }
	    print '<tr><th> Comments: </th><td>';
	    print textarea(-name=>'comments',-id=>'comments',-value=>'',-class=>'textfield',-rows=>8);
	    print '</td></tr>';
	    print '<tr><th> (*) Indicates a required field </th><td>';
	    print button(-name=>'Start program',-class=>'start_program',-onclick=>'start_program()',-id=>'start');
	    print '</td></tr>';
	    print '</table>';
	    print '</div>';
		print '</div>';
		print '<div class="container_div right">';
		print '<div class="furnace_div">';
	    print '<td>',"\n";
	    
	
	    print '<img class=program_img id=profile src="/RFCfurnace/dummy.png">';
	    print '</td>';
	    print '<td style="width:300px;">',"\n";
	    print '</div>';
	    
	    print '</td></tr>',"\n";
	}
	elsif ($readmode)
	{
	    print h1({-class=>'Alert'},'You are not allowed to start or stop furnaces from the current client system or start of furnaces have been disabled by the administrators');
	    print '<table class="furnace_main">',"\n";	    	    
	}
	else
	{
	    print h1({-class=>'Alert'},'You are not allowed to use this furnace');
	    print '<table class="furnace_main">',"\n";	    
	}
	

	#print '</table>';
    }
    if ($modifymode)
    {
	print p();
	if ($furnace->is_running())
		{print '<div class="furnace_div" style="width:45%">';}
	else{print '<div class="furnace_div">';}	
	print '<div>';
	print '<h2 style="margin:0">Admin panel </h2>';
	print '</div>';
	print '<div>';
	print '<p> As furnace administrator you can set temperature and ramp rate directly as well as manage which chemical elements the furnace has experienced:</p>'; 
	print '</div>';	
	print '<div class="row">';
	print '<div class="col-35">';
	print start_form(-name=>'new_temp');
	print '<label for="furnace_setpoint">Furnace set point: </label>';
	print '</div>';
	print '<div class="col-65">';
	print textfield(-name=>'temp',-value=>$temp,-force=>1,-class=>'testfireld',-onchange=>'set_temp(this.value)');
	&action('set_temp');
	print_hidden_rig();
	print end_form();
	print '</div>';
	print '</div>';
	print '<div class="row">';
	print '<div class="col-35">';
	print start_form(-name=>'new_ramp');
	print '<label for="furnace_ramprate">Furnace ramp rate: </lable>';
	print '</div>';
	print '<div class="col-65">';
	print textfield(-name=>'ramp',-value=>$ramp,-force=>1,-class=>'testfireld',-onchange=>'set_ramp(this.value)');
	&action('set_ramp');
	print_hidden_rig();
	print end_form();
	print '</div>';
	print '</div>';
	my @list = ();
	my @elements = RFC::Furnace::get_element_data();
	foreach my $z ($furnace->list_elements())
	{
	    foreach my $e (@elements)
	    {
		push @list,$$e{'short'} if ($$e{'Z'} == $z);
	    }
	}
	print start_form(-name=>'set_elements');
	&action('set_elements');
	print hidden(-name=>'f_elements',-value=>join(',',@list),-force=>1,-id=>'f_elements');
	print_hidden_rig();
	print end_form();
	print '<div class="row">';
	print '<div class="col-65">';
	print button(-name=>'Manage chemical element exposure',-class=>'s4',-onclick=>"pick_elements_furnace(document.getElementById('f_elements').value)") unless (@tubes);
    print '</div>';
    print '</div>';
    }
    print "</table>\n";
    print '</div>';
  
   # print '</center>';
    print start_form(-name=>'data');
    print hidden(-name=>'isactive',-value=>1,-force=>1);
    print_hidden_rig();
    print end_form();
    my $ssl = 'false';
    $ssl = 'true' if (CGI::https());
    my $user = &get_user();
    my @elist = ();
    for my $e (&RFC::Furnace::get_element_data)
    {
	push @elist,' e['.$$e{'Z'}.'] = "'.$$e{'short'}.'";';
    }
    my $old_prog_str;
    if (param('old_program'))
    {
	my @lines = split("\n",param('old_program'));
	pop @lines;
	shift @lines;
	my $count = 0;
	my @prog = ();
	for (my $x = 0; $x < @lines; $x++)
	{
	    $lines[$x] =~ s/[\n\r]//g;
	    next unless ($lines[$x] =~ /temp|hold|ramp\b/);
	    push @prog, "program[".$count++."] = '".$lines[$x]."';";
	}
	$old_prog_str = join("\n",@prog)
    }
    my $project_regex = get_cv('RFCfurnace','project_regex');
    $project_regex = '' unless ($project_regex);
    my $use_project_regex = 'true';
    if ($project_regex)
    {
	$project_regex = '/'.$project_regex.'/i';
    }
    else
    {
	$project_regex = 'false';
	$use_project_regex = 'false';
    }
    my $sstr = join("\n",@elist);
    my ($u,$p) = &RFC::RFCCGI::get_id();
    my $baseimg = '/cgi-bin/show_img.cgi?name='.$u.'&pass='.$p.'&url=http://'.hostname;
    $baseimg = '/cgi-bin/show_img.cgi?name='.$u.'&pass='.$p.'&url=https://'.hostname if (CGI::https());
    print <<END;

<script>

window.onblur = set_inactive;
window.onfocus = set_active;
var ispopup = false;
var is_logged_in = true;
var username = '$user';
var program = new Array;
$old_prog_str
var totalt = 0;
var element_ok = true;
var servers = new Array;
var s_elements = new Array;
var runnr = $runnr;
var active = false;
var is_ssl = $ssl;
var semafore = 0;
var tube_elements = new Array;
var e = new Array();
var rig = $r
$sstr

var minramp = $minramp;
var maxramp = $maxramp;
var mintemp = $mintemp;
var maxtemp = $maxtemp;

var proj_regex = $project_regex;
var use_proj_reg = $use_project_regex;

function show_tube_elements()
{
    var myWindow=window.open('pick_elements.cgi?readonly=1&elements='+tube_elements.join(','),'windowRef','width=1800,height=900,scrollbars=1,resizable=1');
    myWindow.location.href = 'pick_elements.cgi?readonly=1&elements='+tube_elements.join(',');
    if (!myWindow.opener)
        myWindow.opener = self;
    myWindow.focus();
}

function show_furnace_elements()
{
    var myWindow=window.open('pick_elements.cgi?furnace=1&readonly=1&elements='+document.getElementById('f_elements').value,'windowRef','width=1800,height=900,scrollbars=1,resizable=1');
    myWindow.location.href = 'pick_elements.cgi?furnace=1&readonly=1&elements='+document.getElementById('f_elements').value;
    if (!myWindow.opener)
        myWindow.opener = self;
    myWindow.focus();
}



function get_elements_tube(id)
{
    semafore = 0;
    tube_elements.length = 0;
    document.getElementById('tube_e').className = 'hidden';
    s_elements.length = 0;
    for (var x = 0; x < servers.length; x++)
    {
	semafore++;
	var host = 'http://'+servers[x]+'/cgi-bin/RFCfurnace/ajax.cgi';
	if (is_ssl)
	{
	    host = 'https://'+servers[x]+'/cgi-bin/RFCfurnace/ajax.cgi';
	}
	run_ajax(host,'&action=get_tube_elements&tube='+id,function(res) {
	    var arr = res.split(/\\n/);
	    if (arr[0] == 'OK') 
	    { 
		arr.shift(); 
		if (arr[arr.length-1] == '')
	        {
		    arr.pop();
	        }
		s_elements.push(arr.join(',')); 
		semafore--; 
		tube_ok(); 
		return true;
	    } alert(res);
	});
    }
}

function tube_ok()
{
    if (semafore > 0)
    {
	return false;
    }
    for (var x = 0; x < s_elements.length; x++)
    {
	var list = s_elements[x].split(/,/);
	for (var y = 0; y < list.length; y++)
        {
            var n = e[list[y]];
            if (n != null && n != '' && !duplicate_tube_element(n))
            {
                tube_elements.push(n);
            }
        }
    }
    if (document.getElementById('tube').value > 0)
    {
	if (tube_elements.length > 0)
	{
	    alert('This tube has previously been exposed to the following elements:\\n'+tube_elements.join(','));
	}
	else
	{
	    alert('This tube is not registered as having ever been in use!');
	}
	document.getElementById('tube_e').className = 's2';
    }
}

function duplicate_tube_element(name)
{
    for (var x = 0; x < tube_elements.length; x++)
    {
	if (name == tube_elements[x])
        {
            return true;
        }
    }
    return false;
}

function get_listservers()
{
    run_ajax('ajax.cgi','&action=get_listservers',set_listservers);
}

function set_listservers(res)
{
    var arr = res.split(/\\n/);
    if (arr[0] == 'OK')
    {
	for (var x = 1; x < arr.length ; x++)
	{
	    var host = 'http://'+arr[x]+'/cgi-bin/RFCfurnace/ajax.cgi';
	    if (is_ssl)
	    {
		host = 'https://'+arr[x]+'/cgi-bin/RFCfurnace/ajax.cgi';
	    }
	    run_ajax(host,'&action=get_servers',set_servers);
	}
	return true;
    }
    alert(res);
}

function set_servers(res)
{
    var arr = res.split(/\\n/);
    if (arr[0] == 'OK')
    {
	for (var x = 1; x < arr.length ; x++)
	{
	    servers.push(arr[x])
	}
	return true;
    }
    alert(res);
}


function pick_elements_furnace(s)
{
    var myWindow=window.open('pick_elements.cgi?furnace=1&elements='+s,'windowRef','width=1800,height=900,scrollbars=1,resizable=1');
    myWindow.location.href = 'pick_elements.cgi?furnace=1&elements='+s;
    if (!myWindow.opener)
        myWindow.opener = self;
    myWindow.focus();
}


function pick_elements(s)
{
    var myWindow=window.open('pick_elements.cgi?elements='+s,'windowRef','width=1800,height=900,scrollbars=1,resizable=1');
    myWindow.location.href = 'pick_elements.cgi?elements='+s+'&rig='+rig;
    if (!myWindow.opener)
        myWindow.opener = self;
    myWindow.focus();
}

function check_comp(comp)
{
    run_ajax('ajax.cgi','&action=check_atoms&comp='+comp,set_comp);
}

function set_comp(res)
{
    var arr = res.split("\\n");
    if (arr[0] == 'OK')
    {
        arr.shift();
        var str = arr.join("\\n");
        if (confirm(str))
        {
            element_ok = true;
        }
        else
        {
            element_ok = false;
	    document.getElementById('comp').select();
        }
        
    }
    else
    {
        element_ok = false;
        alert(res);
	document.getElementById('comp').select();
    }
}

function get_info()
{
    var info = new Array;
    var ob = document.getElementById('order').value;
    if (ob == '')
    {
	alert('Ordered by not specified!');
	return '';
    }
    info.push('Ordered by: '+ob);
    var tube = document.getElementById('tube').value;
    if (tube == '')
    {
	alert('You have to specify which furnace tube you are using!');
	return '';
    }
    var comp = document.getElementById('comp').value;
    if (comp == '' || element_ok == false)
    {
	alert('Composition not specified!');
	return '';
    }
    if (tube !=  '' && tube != -1)
    {
	info.push('Tube         : '+tube);
    }
    info.push('Composition  : '+comp);
    var id = document.getElementById('id').value;
    info.push('Sample ID    : '+id);
    var project = document.getElementById('project').value;
    info.push('Project      : '+project);
    if (use_proj_reg)
    {
	if (!proj_regex.test(project))
	{
	    alert('Project number not supplied or not correct format!');
	    return '';
	}
    }
    var comm = document.getElementById('comments').value;
    info.push('Comments     : '+comm);
    return info.join('\\n');
}


function stop_program()
{
    if (active)
    {
	return false;
    }
    active = true;
    if (confirm("Are you sure that you want to terminate the running program for $furnace"))
    {
	document.getElementById('stop').className = 'start_inactive';
	if (confirm('Set furnace temperature setpoint to 25 C'))
	{
	    run_ajax('ajax.cgi','&action=stop_program&setpoint=25',function(res){alert(res);document.data.submit()});
	}
	else
	{
	    run_ajax('ajax.cgi','&action=stop_program',function(res){alert(res);document.data.submit()});
	}
    }
}

function start_program()
{
    if (active)
    {
	return false;
    }
    if (program.length == 0)
    {
	alert('No program found\\nNothing to do!');
	return false;
    }
    var info = get_info();
    if (info == '')
    {
	alert('One or more pieces of required information regarding the sample is missing. The furnace can not be started!');
	return false;
    }
    var runmode = '';
    var e = document.getElementById('runmode');
    if (e != null)
    {
	var val = e.value;
	if (val == null || val == '')
	{
	    alert('You have to specify the furnace atmosphere!');
	    return false;
	}
	runmode = val;
    }
    if (totalt > 0)
    {
	active = true;
	info = info.replace(/\\n/g,'NEWLINE');
	var prog = program.join("NEWLINE");
	var comp = document.getElementById('comp').value;
	var tube = document.getElementById('tube').value;
	var te = tube_elements.join(','); 
	document.getElementById('start').className = 'start_inactive';
	run_ajax('ajax.cgi','&action=start_program&program='+prog+'&comp='+comp+'&tube='+tube+'&tube_elements='+te+'&runmode='+runmode+'&info='+info,function(res){alert(res);document.data.submit()});
    }
    else
    {
	alert('Total Program time is 0, nothing to do!');
    }
}

function change_program(p)
{
    if (active)
    {
	return false;
    }
    active = true;
    program = p.split("\\n");
    update_plot();
}

function add_line_hold()
{
    if (active)
    {
	return false;
    }
    active = true;
    var time = document.getElementById('wait').value;
    if (time != '')
    {
	time.replace(/,/g,'.');
	document.getElementById('wait').value = '';
	program.push('hold = '+time);
	update_plot();
	return true;
    }
    update_plot();
    return true;    
}

function add_line_ramp()
{
    if (active)
    {
	return false;
    }
    active = true;
    var ramp = document.getElementById('ramprate').value;
    var temp = document.getElementById('setpoint').value;
    if (temp != '')
    {
	temp.replace(/,/g,'.');
	if (temp < mintemp)
	{
	    temp = mintemp;
	    alert("You are not allowed to set a temperature less than "+mintemp+" C");
	}
	if (temp > maxtemp)
	{
	    temp = maxtemp;
	    alert("You are not allowed to set a temperature greater than "+maxtemp+" C");
	}
	temp = temp.toString();
	if (ramp == '' || ramp == 0)
	{
	    alert('You have to specify a ramp rate when changing temperature!');
	    active = false;
	    return false;
	}
	else
	{
	    if (ramp < minramp)
	    {
		ramp = minramp;
		alert("You can not set a ramprate less than "+minramp+" C/hour");
	    }
	    if (ramp > maxramp)
	    {
		ramp = maxramp;
		alert("You can not set a ramprate greater than "+maxramp+" C/hour");
	    }
	    ramp = ramp.toString()
	}
	ramp.replace(/,/g,'.');
	document.getElementById('ramprate').value = '';
	document.getElementById('setpoint').value = '';
	program.push('ramp = '+ramp);
	program.push('temp = '+temp);
	update_plot();
	return true;
    }
    update_plot();
    return true;    
}

function add_line()
{
    if (active)
    {
	return false;
    }
    active = true;
    var time = document.getElementById('wait').value;
    var ramp = document.getElementById('ramprate').value;
    var temp = document.getElementById('setpoint').value;
    if (time != '')
    {
	time.replace(/,/g,'.');
	if (temp != '')
	{
	    alert('You can not change temperature while holding!');
	    active = false;
	    return false;	    
	}
	document.getElementById('wait').value = '';
	program.push('hold = '+time);
	update_plot();
	return true;
    }
    if (temp != '')
    {
	temp.replace(/,/g,'.');
	if (temp < mintemp)
	{
	    temp = mintemp;
	    alert("You are not allowed to set a temperature less than "+mintemp+" C");
	}
	if (temp > maxtemp)
	{
	    temp = maxtemp;
	    alert("You are not allowed to set a temperature greater than "+maxtemp+" C");
	}
	temp = temp.toString();
	if (ramp == '' || ramp == 0)
	{
	    alert('You have to specify a ramp rate when changing temperature!');
	    active = false;
	    return false;
	}
	else
	{
	    if (ramp < minramp)
	    {
		ramp = minramp;
		alert("You can not set a ramprate less than "+minramp+" C/hour");
	    }
	    if (ramp > maxramp)
	    {
		ramp = maxramp;
		alert("You can not set a ramprate greater than "+maxramp+" C/hour");
	    }
	    ramp = ramp.toString()
	}
	ramp.replace(/,/g,'.');
	document.getElementById('ramprate').value = '';
	document.getElementById('setpoint').value = '';
	program.push('ramp = '+ramp);
	program.push('temp = '+temp);
	update_plot();
	return true;
    }
    update_plot();
    return true;    
}

function add_setpoint()
{
    if (active)
    {
	return false;
    }
    active = true;
    var time = document.getElementById('wait').value;
    var ramp = document.getElementById('ramprate').value;
    var temp = document.getElementById('setpoint').value;
    program.push('ramp = '+ramp);
    program.push('temp = '+temp);
    update_plot()
}


function add_wait()
{
    if (active)
    {
	return false;
    }
    active = true;
    var time = document.getElementById('wait').value;
    var ramp = document.getElementById('ramprate').value;
    var temp = document.getElementById('setpoint').value;
    program.push('hold = '+time);
    update_plot()
}

function update_plot()
{
    if (runnr == 0)
    {
	var prog = program.join("NEWLINE");
	run_ajax('ajax.cgi','&action=add_program&program='+prog,set_plot);    
    }
    else
    {
	run_ajax('ajax.cgi','&action=plot_program&runnr='+runnr,set_plot);    
    }
}


function set_plot(res)
{
    active = false;
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
	document.getElementById('profile').src = '$baseimg'+arr[1]+'&dummy='+Math.random();
	var p = document.getElementById('program');
	if (p != null)
	{
	    p.value = program.join("\\n");
	}
	document.getElementById('totaltime').innerHTML = 'Current program: Total time '+arr[2]+' hours';
	totalt = parseFloat(arr[2]);
	return true;
    }
    alert(res);
}

function set_temp(t)
{
    if (confirm("Change setpoint to "+t))
    {
	document.new_temp.submit()
    }
}

function set_ramp(t)
{
    if (confirm("Change ramprate to "+t))
    {
	document.new_ramp.submit()
    }
}


function store_comment(str,runnr)
{
    var f = document.new_comment;
    var adr = f.addr.value;
    var id = f.eqid.value;
    if (id == 0 || adr == '')
    {
	return true;
    }
    if (confirm('Append this message to the public remote log book'))
    {
	remote_log(adr,id,str,runnr);
    }
}


function check_status()
{
    raw_ajax('../celltest/ajax.cgi','?action=is_logged_in&name='+username,set_status);
}

function set_status(res)
{
    var reg = /YES/;
    if (!reg.test(res))
    {   
	is_logged_in = false;
	document.logout.submit();
    }
}



function set_active()
{
    document.data.isactive.value = '1';
}

/* Ugly hack for IE wrongly triggering window onblur when selecting element inside page..... */

function ie_inactive()
{
    if (bIsMSIE && (active_element != document.activeElement)) {
	active_element = document.activeElement;
    }
    else 
    {	    
	document.data.isactive.value = '0';
    }
}

function set_inactive()
{
    if (navigator.appName == "Microsoft Internet Explorer") {
	active_element = document.activeElement;
	bIsMSIE = true;
	document.onfocusout = ie_inactive;
	return false;
    }
    ie_inactive();
}

/* end IE hack */


function add_comment(runnr)
{
    var text = document.new_comment.comments.value;
    if (text != null && text != "")
    {
	document.new_comment.comments.value = '';
	/* Translate all newlines to literal '\n' strings to make them
	    survive the XHTML transfer..... */
	text = text.replace(\/\\n\/mg,"\\\\n");
	run_ajax('ajax.cgi','&action=add_info&comments='+text,set_info);
	store_comment(text,runnr);
    }
}


function load_log()
{
    if (is_logged_in)
    {
        check_status();
        run_ajax('ajax.cgi','&action=get_proglog',set_log);
        setTimeout(load_log,50000);
    }
}

function load_info()
{
    if (is_logged_in)
    {
        check_status();
        run_ajax('ajax.cgi','&action=get_info',set_info);
        setTimeout(load_info,50000);
    }
}

function load_log_info()
{
    var proglog = document.getElementById('log');
    if (proglog == null)
    {
	return false;
    }
    load_log();
    load_info();
}


function set_log(t)
{
    var proglog = document.getElementById('log');
    proglog.value = t;
    proglog.scrollTop = proglog.scrollHeight;
}

function set_info(t)
{
    document.getElementById('info').value = t;
}



update_plot();
load_log_info();
get_listservers();


</script>

END


}    

