#!/usr/bin/perl
# $Id: search.cgi,v 1.12 2018/03/16 06:51:56 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2017              ###
###                                                      ###
############################################################
############################################################
use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use RFC::Furnace;
use POSIX;
my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
my $furnace = undef;
my $equipid = $rig->get_cv('main','equipmentid');
$equipid = 0 unless ($equipid);
my $eqadr = get_cv('global','remote_equipment_database');
$eqadr = '' unless ($eqadr);
my $logaddr = get_cv('global','logbook_url');
my $number_of_entries_page = 10;

if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end('../RFCfurnace/manual.pdf');
}
else
{
    if (login_ok())
    { 
        &print_header("Furnace runs for furnace $rig");
	print RFC_start_html(-title=>"Furnace runs for furnace $rig", -style=>{src=>'/RFCfurnace/Furnace.css'});
        top_nav_bar_start;
        if ($logaddr && $equipid)
        {
            top_nav_bar_button($logaddr,'','','Logbook for furnace '.$rig->name,'s3','rig,'.$rig->name.',1',"id,$equipid,1");
        }
        else
        {
            top_nav_bar_button('../celltest/logbook.cgi','','','Logbook for furnace '.$rig->name,'s3','rig,'.$rig->name.',1');
        }
        top_no_button;
	top_nav_bar_button('furnace.cgi','','','Furnace '.$rig->name.' main page','s2','rig,'.$rig->name.',1');
        tab_newrow;
        if (get_CGI_value('action'))
        {
            top_no_button;
        }
        else
        {
            &js_back();
            top_js_return('#006600','white');
        }
        top_no_button;
	print '<td>';
        print start_form(-action=>'/rig'.$r.'/furnace_runs',-method=>'GET');
        menu_button('test',"Raw data",'s7');
        print end_form();
	print '</td>';
        top_no_button;
        top_nav_bar_button('main.cgi','select_server','','To menu','s4');
        end_top_bar;
        eval {
            $furnace = RFC::Furnace->new($rig);
        };
        if ($@)
        {
            print '<center>';
            print h1({-class=>'Alert'},$@);
            print h1("You likely have to configure the underlying RFC::Rig instance to correct this problem");
            print '</center>';
        }
        else
        {
            param(-name=>'action',-value=>'')  unless (defined(param('action')));
            $_ = param('action');
	    CASE:
            {
                ### Insert case statements ###
                &main_page;
            }
        }
        &print_end('../RFCfurnace/manual.pdf');
    }
    else 
    {
        &not_auth;
    }
}

$rig = undef;    
exit;


sub main_page
{
    print '<center>';
    my @tests = $furnace->list_runs();
    my $page = param('page');
    $page = 0 unless ($page);
    my $totalpages = POSIX::floor(@tests/$number_of_entries_page);
    ### Fix offset for 'even' number of tests ###
    $totalpages-- if (@tests && ! (@tests%$number_of_entries_page));
    my $startnr = ($totalpages - $page) * 10;
    my $end = @tests;
    $end = $startnr + 10 if ($page);
    print '<table class=search><tr><td>';
    if ($page < $totalpages)
    {
	print start_form();
	&print_hidden_rig();
	my $val = $page + 1;
	print hidden(-name=>'page',-value=>$val,-force=>1);
	print submit(-name=>'action',-value=>'<',-class=>'search_nav');
	print end_form();
    }
    print '</td><th>';
    print h2('Showing furnace runs '.$tests[$startnr].' to '.$tests[$end-1]." for $furnace");
    print '</th><td>';
    if ($page)
    {
	print start_form();
	&print_hidden_rig();
	my $val = $page - 1;
	print hidden(-name=>'page',-value=>$val,-force=>1);
	print submit(-name=>'action',-value=>'>',-class=>'search_nav');
	print end_form();
    }
    print '</td></tr></table>';
    print '<table width=60%>';
    print '<tr>';
    print '<th> Furnace run </th>';
    print '<th> Start date </th>';
    print '<th> Started by </th>';
    print '<th> Composition </th>';
    print '<th> Comments </th>';
    print '</tr>';
    for (my $x = $startnr; $x < $end; $x++)
    {
	my %info = $furnace->get_run_info($tests[$x]);
	print '<tr><td>';
	print start_form(-action=>'show_old.cgi');
	&print_hidden_rig();
	print hidden(-name=>'testnr',-value=>$tests[$x],-force=>1);
	print submit(-name=>'dummy',-value=>'Furnace run '.$tests[$x],-class=>'s3');
	print end_form();
	print '</td><th>';
	print xss($info{'start_date'});
	print '</th><th>';
	print xss($info{'started_by'});
	print '</th><th>';
	print xss($info{'composition'});
	print '</th><th>';
	my $str = xss($info{'comments'});
	my @l = split(/\n+/,$str);
	my @l2 = @l[0..1];
	$str = join('<br>',@l2);
	print $str;
	print ' .... ' if (@l > 2);
	print '</th></tr>';
    }
    print '</table>';
    print '</center>';
}
