#!/usr/bin/perl                                                                 
# $Id: show_old.cgi,v 1.16 2023/05/25 12:14:26 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2017              ###
###                                                      ###
############################################################
############################################################
use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use Sys::Hostname;
use RFC::Furnace;
my $f = get_CGI_value_clean('rig');
my $run = get_CGI_value_clean('runnr');
$run = get_CGI_value_clean('testnr') unless ($run);
unless ($f)
{
    $f = get_CGI_value_clean('furnace');
    param(-name=>'rig',-value=>$f);
}
unless ($f)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($f);
my $equipid = $rig->get_cv('main','equipmentid');
$equipid = 0 unless ($equipid);
my $eqadr = get_cv('global','remote_equipment_database');
$eqadr = '' unless ($eqadr);
my $logaddr = get_cv('global','logbook_url');

### Check for data access ##
my $hostname = hostname();
my ($u,$p) = &RFC::RFCCGI::get_id();
my $allow = 0;
my $uid = 0;
my @authids = &RFC::Main::get_cv('global','data_access_authorisationid');
if (@authids)
{
    $uid = RFC::RFCCGI::login_ok();
    if ($uid)
    {
        foreach my $id (@authids)
        {
            ## Allow is user is authorised for at least one of the listed authorisations
            next if ($allow);
            $allow = &RFC::Main::passwd_client('safety_auth',$u,$id);
        }
    }
    else
    {
        $allow = 0;
    }    
}
else
{
    $allow = 1;
}

unless ($allow)
{
    print header("Data gateway for $hostname");
    print start_html("Data gateway for $hostname");
    print '<link rel="stylesheet" type="text/css" href="/RFCfurnace/Furnace.css" />',"\n";
    print center(h1('Warning: You are not allowed to access data on this server!'));
    if ($uid)
    {
        print center(h1("You likely need authroisation for one of the following authorisations: ".join(', ',@authids)));
    }
    else
    {
        print center(h1('Possibly your session has timed out'));
    }   
    print start_form(-action=>'/cgi-bin/RFCfurnace/prelogin.cgi');
    print center(submit(-name=>'dummy',-value=>'To login page',-class=>'s3',-style=>'width:200px;'));
    print end_form();
    print end_html();
    exit;
}

## Contibue if ok ###

if (param('action') && param('action') eq 'print_page')
{
    print header();
    print start_html();
    $rig = RFC::Furnace->new($rig);
    print <<END;

<link rel="stylesheet" type="text/css" href="/RFCfurnace/Furnace.css" />
<script src="/celltest/RFCcontrol.js" type="text/javascript"></script>

END

    print start_form(-name=>'login_info');
    print_hidden();
    print hidden(-name=>'rig',-value=>$rig->name,-force=>1);
    print hidden(-name=>'runnr',-value=>$run,-force=>1);
    print hidden(-name=>'time');
    print hidden(-name=>'jscheck');
    print end_form;
    &show_old($run,'print');
    &print_end('../RFCfurnace/manual.pdf');
    exit;
}


if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end('../RFCfurnace/manual.pdf');
}
else
{
    $rig = RFC::Furnace->new($rig);
    if (login_ok())
    {
        &print_header("Old furnace run for ".$rig,-onload=>'load_start()');
        print <<END;

<script>

function load_start()
{

}

</script>

END

        print RFC_start_html(-title=>"Main menu for rig ".$rig->name, -style=>{src=>'/RFCfurnace/Furnace.css'});
        top_nav_bar_start;
        if ($logaddr && $equipid)
        {
	    top_nav_bar_button($logaddr,'','','Logbook for furnace '.$rig->name,'s3','rig,'.$rig->name.',1',"id,$equipid,1");
	}
        else
        {
	    top_nav_bar_button('../celltest/logbook.cgi','','','Logbook for furnace '.$rig->name,'s3','rig,'.$rig->name.',1')
	}
        top_no_button;
        top_no_button;
        tab_newrow;
        if (get_CGI_value('action') && get_CGI_value('action') !~ /show_details/i)
        {
	    top_no_button;
	}
        else
        {
	    &js_back();
	    top_js_return('#006600','white');
	}
        top_no_button;
        top_nav_bar_button('','print_page','','Printer friendly print','s4','rig,'.$rig->name.',1','runnr,'.$run.',1');
        top_no_button;
        top_nav_bar_button('main.cgi','select_server','','To menu','s4');
        end_top_bar;
        &show_old($run);
        &print_end('../RFCfurnace/manual.pdf');
    }
    else
    {
        &not_auth;
    }
}

sub show_old($)
{
    my $runnr = shift;
    my $print = shift;
    my $htacc = $rig->testdir($runnr).'/.htaccess';
    print '<center>';
    if (-e $htacc)
    {
	print h1("Warning the requested furnace run is subject to access restrictions.");
	print h2('click the link below to access the data (requires password)');
	print "< href='/rig".$f."/furnace_runs/$runnr'> Furnace run $runnr </a>";
	print '</center>';
	return;
    }
    my $furnace = RFC::Furnace->new($rig);
    my @p = $furnace->get_program($runnr);
    my $prog = join("\n",@p);
    my @fp = ();
    foreach my $i (@p)
    {
	push @fp,$i  if ($i =~ /^ramp\b|^temp|hold/);
    }
    my $user = &get_user();
    my ($u,$p) = &RFC::RFCCGI::get_id();
    my $baseimg = '/cgi-bin/show_img.cgi?name='.$u.'&pass='.$p.'&url=http://'.hostname;
    $baseimg = '/cgi-bin/show_img.cgi?name='.$u.'&pass='.$p.'&url=https://'.hostname if (CGI::https());
    my $basedata = '/cgi-bin/show_data.cgi?name='.$u.'&pass='.$p.'&url=http://'.hostname;
    $basedata = '/cgi-bin/show_data.cgi?name='.$u.'&pass='.$p.'&url=https://'.hostname if (CGI::https());
    my $fprog = join("\n",@fp);
    print h2("Information for run $runnr for $rig!");
    print '<table class="furnace_main">',"\n";
    print '<tr><td>',"\n";
    print '<table class="program">';
    print '<tr><th>';
    print 'Program information:</th></tr>';
    if ($print)
    {
	print '<tr><td id=info>';
    }
    else
    {
	print '<tr><td>';
	print textarea(-name=>'info',-id=>'info',class=>'proglog',-rows=>9,-readonly=>1);
    }
    print '</td></tr>';
    print '<tr><th>';
    print 'Furnace program:</th></tr>';
    if ($print)
    {
	$fprog =~ s/\n/<br>/g;
	print '<tr><td id=proglog>'.$fprog;	
    }
    else
    {
	print '<tr><td>';
	print textarea(-name=>'program',-id=>'proglog',class=>'proglog',-rows=>5,-readonly=>1,-value=>$fprog);
    }
    print '</td></tr>';
    if ($print)
    {
	print '<tr><td>';
	print '<a href="'.$basedata.'/rig'.$f.'/furnace_runs/'.$runnr.'/data.txt"><img id="profile" src="/RFCfurnace/dummy.png"></img></a>';	
	print '</td></tr>';
    }
    print '<tr><th>';
    print 'Program log:</th></tr>';
    if ($print)
    {
	print '<tr><td id=log>';
    }
    else
    {
	print '<tr><td>';
	print textarea(-name=>'log',-id=>'log',class=>'proglog',-rows=>5,-readonly=>1);
    }
    print '</td></tr>';
    if ($print)
    {
	print '<tr><td>';
    }
    else
    {
	print '</table>';
	print '</td><td>',"\n";
	print '<a href="'.$basedata.'/rig'.$f.'/furnace_runs/'.$runnr.'/data.txt"><img id="profile" src="/RFCfurnace/dummy.png"></img></a>';
    }
    print '</td></tr>',"\n";
    print "</table>\n";
    if ($print)
    {
	print '</td></tr>';
	print '</table>';
    }
    else
    {
	print start_form(-action=>'furnace.cgi');
	&print_hidden();
	print hidden(-name=>'old_program',-value=>$prog,-force=>1);
	print hidden(-name=>'rig',-value=>$f,-force=>1);
	print submit(-name=>'action',-value=>'Use this furnace program',-class=>'s3',-style=>'width:300px;');
	print end_form();
    }
    print '</center>';
    my $isprint = 'false';
    $isprint = 'true' if ($print);
    print <<END;

<form name=data>
<input type=hidden name=isactive value=0>
</form>


<script>

window.onblur = set_inactive;
window.onfocus = set_active;
var ispopup = false;
var is_logged_in = true;
var username = '$user';
var program = new Array;
var totalt = 0;
    var isprint = $isprint;

var runnr = $runnr;
var active = false;

function print_page()
{
    window.print();
    history.back();
}


function update_plot()
{
    run_ajax('ajax.cgi','&action=plot_program&runnr='+runnr,set_plot);
}

function set_plot(res)
{
    active = false;
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
        document.getElementById('profile').src = '$baseimg'+arr[1]+'&dummy='+Math.random();
        var p = document.getElementById('program');
        if (p != null)
        {
            p.value = program.join("\\n");
        }
/*        document.getElementById('totaltime').innerHTML = 'Current program: Total time '+arr[2]+' hours';
        totalt = parseFloat(arr[2]); */
	if (isprint)
	{
	    setTimeout(print_page,3000);
	}
	return true;
    }
    alert(res);
}

function check_status()
{
    raw_ajax('../celltest/ajax.cgi','?action=is_logged_in&name='+username,set_status);
}

function set_status(res)
{
    var reg = /YES/;
    if (!reg.test(res))
    {
        is_logged_in = false;
        document.logout.submit();
    }
}

function set_active()
{
    document.data.isactive.value = '1';
}

/* Ugly hack for IE wrongly triggering window onblur when selecting element inside page..... */

function ie_inactive()
{
    if (bIsMSIE && (active_element != document.activeElement)) {
        active_element = document.activeElement;
    }
    else
    {
        document.data.isactive.value = '0';
    }
}

function set_inactive()
{
    if (navigator.appName == "Microsoft Internet Explorer") {
        active_element = document.activeElement;
        bIsMSIE = true;
        document.onfocusout = ie_inactive;
        return false;
    }
    ie_inactive();
}

/* end IE hack */

function load_log()
{
    if (is_logged_in)
    {
        check_status();
        run_ajax('ajax.cgi','&action=get_proglog&runnr='+runnr,set_log);
    }
}

function load_info()
{
    if (is_logged_in)
    {
	check_status();
	run_ajax('ajax.cgi','&action=get_info&runnr='+runnr,set_info);
    }
}

function set_log(t)
{
    if (isprint)
    {
	var proglog = document.getElementById('log');
	proglog.innerHTML = t.replaceAll('\\n','<br>');
    }
    else
    {
	var proglog = document.getElementById('log');
	proglog.value = t;
	proglog.scrollTop = proglog.scrollHeight;
    }
    load_info();
}

function set_info(t)
{
    if (isprint)
    {
	document.getElementById('info').innerHTML = t.replaceAll('\\n','<br>');
    }
    else
    {
	document.getElementById('info').value = t;
    }
    update_plot();
}

load_log();

</script>

END

}
